package xyz.lychee.lagfixer.libs.adventure.internal.properties;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.LinkOption;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Optional;
import java.util.Properties;
import java.util.function.Function;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.VisibleForTesting;

final class AdventurePropertiesImpl {
   private static final String FILESYSTEM_DIRECTORY_NAME = "config";
   private static final String FILESYSTEM_FILE_NAME = "adventure.properties";
   private static final Properties PROPERTIES = new Properties();

   private static void print(final Throwable ex) {
      ex.printStackTrace();
   }

   private AdventurePropertiesImpl() {
   }

   @VisibleForTesting
   static @NotNull String systemPropertyName(final String name) {
      return String.join(".", "net", "kyori", "adventure", name);
   }

   static <T> AdventureProperties.@NotNull Property<T> property(final @NotNull String name, final @NotNull Function<String, T> parser, final @Nullable T defaultValue) {
      return new PropertyImpl(name, parser, defaultValue);
   }

   static {
      Path path = (Path)Optional.ofNullable(System.getProperty(systemPropertyName("config"))).map((x$0) -> {
         return Paths.get(x$0);
      }).orElseGet(() -> {
         return Paths.get("config", "adventure.properties");
      });
      if (Files.isRegularFile(path, new LinkOption[0])) {
         try {
            InputStream is = Files.newInputStream(path);

            try {
               PROPERTIES.load(is);
            } catch (Throwable var5) {
               if (is != null) {
                  try {
                     is.close();
                  } catch (Throwable var4) {
                     var5.addSuppressed(var4);
                  }
               }

               throw var5;
            }

            if (is != null) {
               is.close();
            }
         } catch (IOException var6) {
            IOException e = var6;
            print(e);
         }
      }

   }

   private static final class PropertyImpl<T> implements AdventureProperties.Property<T> {
      private final String name;
      private final Function<String, T> parser;
      private final @Nullable T defaultValue;
      private boolean valueCalculated;
      private @Nullable T value;

      PropertyImpl(final @NotNull String name, final @NotNull Function<String, T> parser, final @Nullable T defaultValue) {
         this.name = name;
         this.parser = parser;
         this.defaultValue = defaultValue;
      }

      public @Nullable T value() {
         if (!this.valueCalculated) {
            String property = AdventurePropertiesImpl.systemPropertyName(this.name);
            String value = System.getProperty(property, AdventurePropertiesImpl.PROPERTIES.getProperty(this.name));
            if (value != null) {
               this.value = this.parser.apply(value);
            }

            if (this.value == null) {
               this.value = this.defaultValue;
            }

            this.valueCalculated = true;
         }

         return this.value;
      }

      public boolean equals(final @Nullable Object that) {
         return this == that;
      }

      public int hashCode() {
         return this.name.hashCode();
      }
   }
}
